//-----------------------------------------------------------------------------
// Author:Jonus					Homepage: http://www.chronocross.de/directx.htm
//
// 2nd Author: Joachim			Homepage: http://www.crowsoft.de
//
// File: cubeworld.cpp			Date: 21.01.2001
//
// Desc: It shows how u can make a 3d-map out of two int array
//		 Improved by Joachim - thx. 
//-----------------------------------------------------------------------------

#include <d3dx8.h>
#include <mmsystem.h>

// Our custom FVF, which describes our custom vertex structure
/**
 * You dont need a diffuse, so better not use it for speed reasons.
 */
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|/*D3DFVF_DIFFUSE|*/D3DFVF_TEX1)

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D8				g_pD3D			= NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE8		g_pd3dDevice	= NULL; // Our rendering device
LPDIRECT3DVERTEXBUFFER8	g_pVBMap		= NULL; // Buffer to hold map vertices
LPDIRECT3DTEXTURE8		g_pTilesTexture	= NULL; // Map texture

/**
 * The number of Polygons to draw
 */
DWORD g_dwNumOfPolygons = 0;

// A structure for our custom vertex type
struct CUSTOMVERTEX
{
    D3DXVECTOR3 position;	// The untransformed, 3D position for the vertex
    D3DXVECTOR3 normal;   
    /**
	 * The color in your program is obsolete and thus only requires memory and
	 * time.
	 */
	//D3DCOLOR color;			// The vertex color
	FLOAT tu, tv;			// The texture coordinates
};

typedef struct TEXTURETILES
{
	FLOAT x1,x2,y1,y2;
} t_TexCor;

t_TexCor tex_tile[30];

int map[20][20];
int map_heigth[20][20];

// **********************************************************
void randomise_array(int * p_do_it,int modulo,int multi)
{
for(int i=0;i<20*20;i++)
{
	p_do_it[i] = multi * ((timeGetTime()+rand())%modulo);
}	
}

// Function to create texture coordinates in tex_tile
/**
 * This is not completely correct for DirectX. You see artifacts if you
 * do so in DX8 (with bilinear filtering). You must point the coordinates
 * to the centers of the texels, not their borders.
 * I will suppose your texture has a resolution of 512x512. For lower
 * resolutions you must change this.
 */
#define TEXTURE_RESOLUTION 512.0f
void createtiles(int x_anz, int y_anz, t_TexCor * destination)
{
	int counttiles=0;
	float texeladjust = 1.0f / TEXTURE_RESOLUTION;
	
	for(int i=0;i<y_anz;i++){
		for(int j=0;j<x_anz;j++){
			destination[counttiles].y1=(float)i*1/y_anz + texeladjust;
			destination[counttiles].x1=(float)j*1/x_anz + texeladjust;
			destination[counttiles].y2=(float)(1+i)*1/y_anz - texeladjust;
			destination[counttiles].x2=(float)(1+j)*1/x_anz - texeladjust;
			counttiles++;
		}
	}
}

/**
 * Shot away the diffuse stuff.
 */
void createsquare(D3DXVECTOR3 D3D_P1,D3DXVECTOR3 D3D_P2,
				  D3DXVECTOR3 D3D_P3,D3DXVECTOR3 D3D_P4,
				  CUSTOMVERTEX * pVer,int buffer_s,int ptiles)
{
	D3DXVECTOR3 temp_sol,temp_1,temp_2;
	temp_1 = D3D_P1 - D3D_P2;
	temp_2 = D3D_P1 - D3D_P3;
	D3DXVec3Cross(&temp_sol, &temp_1, &temp_2);

	pVer[buffer_s+0].position=D3D_P1;
	//pVer[buffer_s+0].color=0xffffffff;
	pVer[buffer_s+0].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+0].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+0].normal = temp_sol;
	pVer[buffer_s+1].position=D3D_P3;
	//pVer[buffer_s+1].color=0xffffffff;
	pVer[buffer_s+1].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+1].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+1].normal = temp_sol;
	pVer[buffer_s+2].position=D3D_P2;
	//pVer[buffer_s+2].color=0xffffffff;
	pVer[buffer_s+2].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+2].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+2].normal = temp_sol;

	pVer[buffer_s+3].position=D3D_P3;
	//pVer[buffer_s+3].color=0xffffffff;
	pVer[buffer_s+3].tu= tex_tile[ptiles].x2;
	pVer[buffer_s+3].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+3].normal = temp_sol;
	pVer[buffer_s+4].position=D3D_P1;
	//pVer[buffer_s+4].color=0xffffffff;
	pVer[buffer_s+4].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+4].tv= tex_tile[ptiles].y1;
	pVer[buffer_s+4].normal = temp_sol;
	pVer[buffer_s+5].position=D3D_P4;
	//pVer[buffer_s+5].color=0xffffffff;
	pVer[buffer_s+5].tu= tex_tile[ptiles].x1;
	pVer[buffer_s+5].tv= tex_tile[ptiles].y2;
	pVer[buffer_s+5].normal = temp_sol;
}

// Function to create cubes
/**
 * Changed to avoid (some) useless polygons.
 * The adjustment with .01 is necessary due to numerical errors (<= impossible)
 */
void createcube(float x,float y,float z,
				CUSTOMVERTEX * pVer,int &vbuffer, int tex_top, int tex_side,
				float hRight, float hLeft, float hFront, float hBack)
{
// Top
createsquare(	D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_top);vbuffer+=6;
// The four sides
if (hBack+.01 < y)
{
	createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
					D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
					D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),
					D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;
}

if (hRight+.01 < y)
{
	createsquare(	D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
					D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
					D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),
					D3DXVECTOR3(1.0f+x,1.0f+y,0.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;
}

if (hFront+.01 < y)
{
createsquare(	D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),
				D3DXVECTOR3(1.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;
}

if (hLeft+.01 < y)
{
createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,0.0f+z),
				D3DXVECTOR3(0.0f+x,1.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;
}
/* Bottom
createsquare(	D3DXVECTOR3(0.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,0.0f+z),
				D3DXVECTOR3(1.0f+x,0.0f+y,1.0f+z),
				D3DXVECTOR3(0.0f+x,0.0f+y,1.0f+z),pVer,vbuffer,tex_side);vbuffer+=6;*/
}

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate8( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Get the current desktop display mode, so we can set up a back
    // buffer of the same format
    D3DDISPLAYMODE d3ddm;
    if( FAILED( g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice
    /**
	 * For debuging, D3DSWAPEFFECT_DISCARD is good, for speed not.
	 * Even though I think discard is used in Debug mode anyways. Well...

	 */
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );

	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

    // Create the D3DDevice
    /**
	 * You might test for HARDWARE_VERTEXPROCESSING. But Id be really
	 * careful.
	 */
	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
        return E_FAIL;
    }

    // Turn off culling, so we see the front and back of the triangle
	/**
	 * Id always turn on culling if you dont see the back of polygons.
	 * This way you will only have to draw +- half as much polygons.
	 */
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );

	// alphablend states
	/**
	 * Why?
	 */
	//g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
	//g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
	//g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

	// Enable alpha testing (skips pixels with less than a certain alpha.)
	/**
	 * I would never use anything like alpha testing or chroma keying.
	 * This potentially kills performence on about half of the graphics cards.
	 * Especially as your alpha will always be on 0xff, or? So why turn it on?
	 */
	/*g_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHAREF,        0x08 );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL );*/

	/**
	 * Id rather suggest to use Bilinear filtering. This means no speed
	 * decrease on modern hardware and allows smaller textures. Even if you
	 * zoom in close (this is only true sometimes, though).
	 * Another note. Some graphics cards (like mine) support no more than
	 * 256x256 textures. So your 512x512 texture is sampled down to 256x256.
	 * Even though I get good results here with bilinear filtering and a
	 * closer zoom than the one you used.
	 */
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );


    // Turn off D3D lighting, since we are providing our own vertex colors
	/**
	 * This is not true, so please do not confuse me :)
	 */
    //g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Creates the scene geometry
//-----------------------------------------------------------------------------
HRESULT InitGeometry()
{
	int counter=0;

	randomise_array( *map,10,1);
	randomise_array( *map_heigth,3,1);

	// Texture for Map
	if( FAILED( D3DXCreateTextureFromFileA( g_pd3dDevice, "SF2Tiless.png",&g_pTilesTexture ) ) )
	return E_FAIL;
	createtiles(4,4,tex_tile);

	// Create the vertex buffer. 
	/**
	 * I changed one flags due to speed reasons (even thought there would
	 * only be a speed increase if you used hardware vertexprocessing):
	 * D3DUSAGE_WRITEONLY 
	 * "Informs the system that the application writes only to the vertex
	 * buffer. Using this flag enables the driver to choose the best memory
	 * location for efficient write operations and rendering. Attempts to
	 * read from a vertex buffer that is created with this capability will
	 * fail."
	 */
	if( FAILED( g_pd3dDevice->CreateVertexBuffer( 5*2*3*20*20*sizeof(CUSTOMVERTEX),
												  D3DUSAGE_WRITEONLY,
												  D3DFVF_CUSTOMVERTEX,
												  D3DPOOL_DEFAULT, &g_pVBMap ) ) )
	{
		return E_FAIL;
	}

	// Fill the vertex buffer
	CUSTOMVERTEX * pmapVertices;
	if( FAILED( g_pVBMap->Lock( 0, 0, (BYTE**)&pmapVertices, 0 ) ) )
		return E_FAIL;

	/**
	 * Changed some stuff (see createcube())
	 * Didnt draw the outmost things...
	 */
	for(int x_pos=1;x_pos<19;x_pos++)
	{
		
		for(int z_pos=1;z_pos<19;z_pos++)
		{
		  createcube((float)x_pos,(float)map_heigth[x_pos][z_pos]/2,(float)z_pos,
				pmapVertices,counter, map[x_pos][z_pos], map[x_pos][z_pos],								
				(float)map_heigth[x_pos+1][z_pos]/2,
				(float)map_heigth[x_pos-1][z_pos]/2,
				(float)map_heigth[x_pos][z_pos+1]/2,
				(float)map_heigth[x_pos][z_pos-1]/2);
		}		
	}

	/**
	 * The number of polygons may be less than 5*2*3*20*20.
	 */
	g_dwNumOfPolygons = counter / 3;

	g_pVBMap->Unlock();

	// For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
	/**
	 * Adjusted near clipping plane for being able to zoom in closer and
	 * far clipping plane to improve ZBuffering.
	 */
    D3DXMATRIX matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.5f, 0.1f, 20.0f );
	//D3DXMatrixPerspectiveLH( &matProj, 1.0f, 1.0f, 1.0f, 100.0f );

    g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

	
	/**
	 * Matrial.
	 */
	// Set up a material. The material here just has the diffuse and ambient
    // colors set to yellow. Note that only one material can be used at a time.
    D3DMATERIAL8 mtrl;
    ZeroMemory( &mtrl, sizeof(D3DMATERIAL8) );
    mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
    mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
    mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
    mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
    g_pd3dDevice->SetMaterial( &mtrl );

	/**
	 * Enable lighting.
	 */
	g_pd3dDevice->LightEnable( 0, TRUE );
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

    // Finally, turn on some ambient light.
    g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00202020 );

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{
	if( g_pTilesTexture != NULL )
		g_pTilesTexture->Release();

    if( g_pVBMap != NULL )
        g_pVBMap->Release();

    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();
}



//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------
VOID SetupMatrices()
{
    // Here is our world matrix.
    /**
	 * Lets not waste time :)
	 */
	//D3DXMATRIX matWorld;
	//D3DXMatrixRotationYawPitchRoll( &matWorld, 0.0f,0.0f,0.0f );// <-- Here u can make world transforms
	//D3DXMatrixRotationZ( &matWorld, 0.0f);
    //g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
	
    // Set up our view matrix. A view matrix can be defined given an eye point(1),
    // a point to lookat(2), and a direction for which way is up(3). 
    
    D3DXMATRIX matView;
    D3DXMatrixLookAtLH( &matView, &D3DXVECTOR3( 10.0f+5*cosf(timeGetTime()/3000.0f), 4.2f+2*cosf(timeGetTime()/3000.0f), 10.0f+5*sinf(timeGetTime()/4000.0f) ),// (1)
                                  &D3DXVECTOR3( 10.5f +4*sinf(timeGetTime()/2000.0f), 0.0f, 10.5f+sinf(timeGetTime()/2000.0f) ),// (2) 
                                  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );// (3)
    g_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

    /**
	 * The projection matrix is set at InitGeometry to be called only
	 * once per start, not once per frame.
	 */
}




//-----------------------------------------------------------------------------
// Name: SetupLights()
// Desc: Sets up the lights and materials for the scene.
//-----------------------------------------------------------------------------
VOID SetupLights()
{
    /**
	 * Matrial is set in InitGeometry for speed reasons.
	 */

    // Set up a white, directional light, with an oscillating direction.
    // Note that many lights may be active at a time (but each one slows down
    // the rendering of our scene). However, here we are just using one. Also,
    // we need to set the D3DRS_LIGHTING renderstate to enable lighting
    D3DXVECTOR3 vecDir;
    D3DLIGHT8 light;
    ZeroMemory( &light, sizeof(D3DLIGHT8) );
    light.Type       = D3DLIGHT_DIRECTIONAL;
	float temp_color = (float) timeGetTime();
    light.Diffuse.r  = (float)0.74f+cosf(temp_color/10000)*0.25f;
    light.Diffuse.g  = 1.0f;
    light.Diffuse.b  = (float)0.9f+cosf(temp_color/20000)*0.1f;
    vecDir = D3DXVECTOR3(10.0f+14.0f*cosf(timeGetTime()/3000.0f),
                         10.0f+ 5.0f*cosf(timeGetTime()/10000.0f),
                         10.0f+14.0f*sinf(timeGetTime()/3500.0f) );
    D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );
    light.Range       = 1000.0f;
    g_pd3dDevice->SetLight( 0, &light );
   
	/**
	 * Enabling light is done in InitGeometry.
	 */
}



//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
    // Clear the backbuffer and the zbuffer
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         D3DCOLOR_XRGB(95,125,255), 1.0f, 0 );

    // Begin the scene
    g_pd3dDevice->BeginScene();

    // Setup the lights and materials
    SetupLights();

    // Setup the world, view, and projection matrices
    SetupMatrices();


    // Setup our texture
	g_pd3dDevice->SetTexture( 0, g_pTilesTexture ); // Now use TilesTexture
    g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    /**
	 * This is not really necessary I think.
	 */
	//g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    //g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    //g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );

    // Render the map vertex buffer contents 
	g_pd3dDevice->SetStreamSource( 0, g_pVBMap, sizeof(CUSTOMVERTEX) );
    g_pd3dDevice->SetVertexShader( D3DFVF_CUSTOMVERTEX );
	
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, g_dwNumOfPolygons );


    // End the scene
    g_pd3dDevice->EndScene();

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

		/**
		 * Change the backbuffer resolution whenever the window is resized.
		 * I did this as I neither like big pixels nor wasted time.
		 * OOps. I didnt check for memory underflow. Well. Lets crash the
		 * machine ;)
		 */
		case WM_SIZE:
			Cleanup();
			InitD3D(hWnd);
			InitGeometry();
			break;

    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "D3D Tutorial", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "D3D Tutorial", "D3D Tutorial 13: Cubeworld",
                              WS_OVERLAPPEDWINDOW, 100, 100, 400, 300,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );

    // Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) )
    {
        // Create the scene geometry
        if( SUCCEEDED( InitGeometry() ) )
        {
            // Show the window
            ShowWindow( hWnd, SW_SHOWDEFAULT );
            UpdateWindow( hWnd );

            // Enter the message loop
            MSG msg;
            ZeroMemory( &msg, sizeof(msg) );
            while( msg.message!=WM_QUIT )
            {                
				if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
                {
                    TranslateMessage( &msg );
                    DispatchMessage( &msg );
                }
                else
					// Here are the Gameloops :)					
					Render();
            }
        }
    }

    // Clean up everything and exit the app
    Cleanup();
    UnregisterClass( "D3D Tutorial", wc.hInstance );
    return 0;
}



